# Contents

* [Introduction](#introduction)
* [Installing nexus-server for production](#installing-nexus-server-for-production)
* [Installing nexus-server for development](#installing-nexus-server-for-development)
* [Building and running nexus clients](#building-and-running-nexus-clients)
* [Building and deploying nexus live exam OS](#building-and-deploying-nexus-live-exam-OS)
* [Running validation tests](#running-validation-tests)
* [Server firewall configuration](#server-firewall-configuration)
* [Creating initial templates](#creating-initial-templates)
* [Copy a template from another server](#copy-a-template-from-another-server)
* [Additional tools](#additional-tools)
* [Building QEMU from source](#building-qemu-from-source)
* [Building remote-viewer from source](#building-remote-viewer-from-source)
* [Server software requirements](#server-software-requirements)

<!-- ============================================================================================================== -->
# Introduction

In this document, we describe how to install **nexus** on a **Ubuntu GNU/Linux** system. Certain points mentionned in this document **only** apply to Ubuntu, so if you're running a different distribution, these points will have to be adapted to suit your specific distribution.

<!-- ============================================================================================================== -->
# Installing nexus-server for production

## Required dependencies

On a Ubuntu 24.04 system:
```sh
sudo apt-get install -y qemu-system-x86 qemu-utils guestfish uuid-runtime git golang-go make gnutls-bin qemu-system-modules-spice
```

On a Ubuntu 22.04 system:
```sh
sudo apt-get install -y qemu-system-x86 qemu-utils guestfish uuid-runtime git golang-go make gnutls-bin
```

## Configuring nexus-server (prod)

First, feel free to modify the server configuration file below. It contains auto-explanatory comments that describe what the various options are for:

- `config/server/nexus.conf`

Certificates are required to ensure encrypted communication (TLS) with nexus clients. The installation script creates the required certificates, but you must still edit a few fields specific to your system in the following files:

1. `config/certs/nexus-server.info` : change these fields to your liking: `organization`, `cn`, `dns_name` and `ip_address`.
1. `config/certs/ca.info` : change the `cn` field accordingly.

## Building and installing nexus-server (prod)

Next, run the following to build and install nexus-server. A `PREFIX` environment variable must be set to specify where nexus-server will be installed. Within that directory, a `nexus-server` directory will be created. Below, we set `PREFIX` to `/usr/local`. This means nexus-server will be installed into `/usr/local/nexus-server`:
```sh
make build_srv
sudo -E make install_prod_srv PREFIX=/usr/local
```

The last command above performs the following operations:

- Creates a `nexus` user and group to run the service.
- Creates the file/directory hierarchy in nexus-server's base directory.
- Creates and signs the certificates.
- Add a crond task required by the service (to change kernel permissions which is required to run `guestfish` as a non-root user).
- Integrates the service with systemd.
- Creates a `users.json` file with user `admin@nexus.org`, featuring all capabilities. Its default password is `12345678`, therefore it is **highly** recommended to change it using a nexus client (see below).

## Running nexus-server (prod)

The installation script creates a systemd service named "nexus-server" which can be started with:
```sh
sudo systemctl start nexus-server
```

## Controlling nexus-server with systemd (prod)

Given nexus-server is a systemd service, it can be controled like any other systemd service:

| Command                               | Description                            |
|---                                    |---                                     |
| `systemctl status nexus-server`       | display the state of the service       |
| `sudo systemctl start nexus-server`   | start the service                      |
| `sudo systemctl stop nexus-server`    | stop the service                       |
| `sudo systemctl restart nexus-server` | restart the service                    |
| `sudo systemctl reload nexus-server`  | reload the service                     |
| `sudo systemctl enable nexus-server`  | enable the service at boot time        |
| `sudo systemctl disable nexus-server` | disable the service at boot time       |
| `journalctl -f --unit nexus-server`   | display and follow nexus-server's logs |

## Updating nexus-server binaries (prod)

```sh
make build_srv
sudo systemctl stop nexus-server
sudo -E make update_srv PREFIX=/usr/local
sudo systemctl start nexus-server
```

## Uninstalling nexus-server (prod)

**WARNING: this will delete ALL installed VMs and templates!**

```sh
sudo systemctl stop nexus-server
sudo -E make uninstall_prod_srv PREFIX=/usr/local
```

In case of errors, it is possible to **force** the uninstallation process by specifying `--ignore-errors` to make:

```sh
sudo -E make `--ignore-errors` uninstall_prod_srv PREFIX=/usr/local
```

<!-- ============================================================================================================== -->
# Installing nexus-server for development

Installing nexus-server for development is targeted towards developers who want to contribute to nexus or test it in a development environment.

## Required dependencies

The required dependencies are identical to [the ones defined here](#installing-nexus-server-for-production).

## Building and installing nexus-server (dev)

First, feel free to modify the server configuration file below. It contains auto-explanatory comments that describe what the various options are for:

- `config/server/nexus.conf`

Next, a `PREFIX` environment variable must be set to specify where nexus-server will be installed. Within that directory, a `nexus-server` directory will be created. Below, we set `PREFIX` to the home directory of the current user. This means nexus-server will be installed into `$HOME/nexus-server`.

```sh
export PREFIX=$HOME
````

If bash is your system shell, you can add the `export` line above to the `.bashrc` file in your home directory. This way, whenever you open a new terminal, the `PREFIX` variable will already be set.

Then, run the following commands to build and install nexus-server for development:
```sh
make build_srv
make install_dev_srv
```

## Running nexus-server (dev)

```sh
make run_srv PREFIX=$HOME
```

## Updating nexus-server binaries (dev)

During development, the server code is constantly modified. To build, update and run the server after a change, simply stop nexus-server by pressing CTRL-C and run:
```sh
make build_srv update_srv run_srv
```

## Uninstalling nexus-server (dev)

**WARNING: this will delete ALL installed VMs and templates!**

```sh
make uninstall_dev_srv
```

<!-- ============================================================================================================== -->
# Building and running nexus clients

## Required dependencies

On a Ubuntu system:
```sh
sudo apt-get install -y golang-go make libglfw3-dev libxcursor-dev libxinerama-dev libxi-dev libxxf86vm-dev upx-ucl
```

## Building nexush and nexus-cli

These three environment variables are required:

- `BIN`: must be set to either `nexush` or `nexus-cli` to indicate what client to build
- `SERVER`: defines two things:
  1) nexus-server ip address; must match the address defined in `config/certs/nexus-server.info`
     - typically 127.0.0.1 for development
  1) nexus-server API port; must match the value defined in `config/server/nexus.conf`
     - 1077 by default
- `CERT`:
  - specifies the public certificate (`ca-cert.pem`) to embed in the client
  
Both nexush and nexus-cli clients are built for multiple OSes (Linux, Windows, Mac) and architectures (amd64, i386 and arm64).

The line below builds nexush for all supported OSes and architectures, where the public certificate (`ca-cert.pem`) is located in `$HOME/nexus-server/certs` and the nexus' server ip is 10.0.2.15 and it listens to port 1077:
```sh
make xbuild_client BIN=nexush CERT=$HOME/nexus-server/certs/ca-cert.pem SERVER=10.0.2.15:1077
```

Below, we build nexus-cli for all supported OSes and architectures, without an embedded certificate, and the server's ip is 127.0.0.1 and it listens to port 1077:
```sh
make xbuild_client BIN=nexus-cli SERVER=127.0.0.1:1077
```

The generated binaries are created in the `build` directory. Here is the corresponding file tree, structured by architectures and OSes:
```
build/
├── 386
│   ├── linux
│   │   ├── nexus-cli
│   │   └── nexush
│   └── windows
│       ├── nexus-cli.exe
│       └── nexush.exe
├── amd64
│   ├── darwin
│   │   ├── nexus-cli
│   │   └── nexush
│   ├── linux
│   │   ├── nexus-cli
│   │   └── nexush
│   └── windows
│       ├── nexus-cli.exe
│       └── nexush.exe
└── arm64
    ├── darwin
    │   ├── nexus-cli
    │   └── nexush
    ├── linux
    │   ├── nexus-cli
    │   └── nexush
    └── windows
        ├── nexus-cli.exe
        └── nexush.exe
```

Although the public certificate is embedded in the client, it is possible to override it at execution time through the `NEXUS_CERT` environment variable. For example:
```sh
NEXUS_CERT=$HOME/nexus-server/certs/ca-cert.pem nexush janedoe@nexus.org
```

## Specifically building for Linux/amd64

This is a specific case that builds a client only for Linux/amd64 platofrm. The required environment variables are the same as in [Building nexush and nexus-cli](#building-nexush-and-nexus-cli).

To only build nexush for Linux/amd64:
```sh
make build_nexush CERT=$HOME/nexus-server/certs/ca-cert.pem SERVER=10.0.2.15:1077
```

Similarly, if you're only interested in building nexus-cli for Linux/amd64 uses the `build_nexus-cli` target instead.

## Specifically running nexush for Linux/amd64

To run nexush with user admin@nexus.org (password is 12345678):
```sh
make run_nexush LOGIN=admin@nexus.org
```

## Running a specific client for a specific OS/architecture

To run nexush for Mac/arm64 with user admin@nexus.org:
```sh
build/arm64/darwin/nexush admin@nexus.org
```

To run nexus-cli for Linux/i386 with user janedoe@nexus.org and with an external public certificate (i.e. not embedded):
```sh
NEXUS_CERT=$HOME/nexus-server/certs/ca-cert.pem build/386/linux/nexus-cli janedoe@nexus.org
```

## Building nexus-exam

This section about nexus-exam is only here for completeness and should only matter to developers.

For now, the only supported combination of OS and architecture for nexus-exam is Linux/amd64.

Similarly to nexush and nexus-cli, nexus-exam requires `SERVER` and `CERT` environment variables (see [Building nexush and nexus-cli](#building-nexush-and-nexus-cli)). Furthermore, two other environment variables are required: `EXAM_USER` and `EXAM_PWD`. These define the credentials for the user used by nexus-exam to connect to the nexus server. On the server side, make sure this user has the `VM_ATTACH_ANY` capability.

Here is an example of how to build nexus-exam:
```sh
make build_nexus-exam CERT=$HOME/nexus-server/certs/ca-cert.pem SERVER=10.0.2.15:1077 EXAM_USER=exam@nexus.org EXAM_PWD='3k0%tjslk$g42'
```

## Running nexus-exam

Given server, port, certificate and credentials are embedded in nexus-exam, running it is simply a matter of executing:
```sh
make run_nexus-exam
```
or:
```sh
build/nexus-exam
```

## Deleting all nexus clients

```sh
make clean_client
```

<!-- ============================================================================================================== -->
# Building and deploying nexus live exam OS

TODO

<!-- ============================================================================================================== -->
# Running validation tests

A variety of tests can be performed to locate potentiel bugs in the code.
These tests expect a template that includes "xubuntu" in its name, otherwise it will spill out errors.
Furthermore, make sure the login you specify below has access to this template and has most capabilities enabled. 

To run the tests, make sure nexus-server for development is running, then execute:
```sh
make tests LOGIN=tests@nexus.org
```

<!-- ============================================================================================================== -->
# Server firewall configuration

On the nexus server, it's a good idea to limit incoming connections to nexus-server only (and also ssh).
The `ufw` tool provides an easy way to configure the Linux kernel firewall. Install it with:

```
sudo apt-get install -y ufw
```

Then, make sure to configure the fiewall to match open ports specified in `nexus-server/config/nexus.conf`. There is the API port and the range of ports for Spice.

Let's assume the API port is 1077 and the range of Spice ports is [42000-43000]. Use `ufw` to add the following firewall rules with:
```
ufw allow 1077/tcp
ufw allow 42000:43000/tcp
```

To open ssh access:
```
ufw allow 22/tcp
```

To remove some firewall rules, list their numbers with:
```
ufw status numbered
```

Then delete a rule by specifying its number (here 42):
```
ufw delete 42
```

<!-- ============================================================================================================== -->
# Creating initial templates

To create VMs, at least one template is required. However, none is provided in the base install.

The `template_creator` script, located in `PREFIX/nexus-server/bin/`, can be used to create new templates from ISO images:
```
template_creator - a tool to create a VM template from an ISO image.
After the OS installation is completed, a tar archive is created in the current directory. It can then be unarchived in the nexus-server's templates directory.

Usage: template_creator name owner vis iso
name    name of the template
owner   email address of the template's owner
vis     visibility of the template, either "public" or "private"
iso     ISO image of the OS to install
```

For instance, this would create a template named "Xubuntu 22.04", owned by janedoe@nexus.org, visible to all users (i.e. `public`) and based on the ISO image located in `/tmp/xubuntu-22.04-desktop-amd64.iso`:
```sh
./template_creator "Xubuntu 22.04" janedoe@nexus.org public /tmp/xubuntu-22.04-desktop-amd64.iso
```

Once the command above is ran, a VM is launched in which you can proceed with the installation of the OS from the ISO image. Install the OS as you would on a physical PC. Once installed, you can beboot (the GUI lets you reboot the VM) and configure the OS to your liking. Once you're happy with the OS, close the VM window and `template_creator` will finish the creation of the template, by producing a `.tar` archive of the freshly created template.

You can now unarchive the archive in `PREFIX/nexus-server/data/templates`, set the proper owner, and delete the archive with (we assume the template archive is `7ac36ad7-a960-4c87-916a-4c1068d19b59.tar`):
```sh
sudo -s
cd PREFIX/nexus-server/data/templates
tar vfx PREFIX/nexus-server/bin/template_creator/7ac36ad7-a960-4c87-916a-4c1068d19b59.tar
chown -R nexus:nexus 7ac36ad7-a960-4c87-916a-4c1068d19b59
rm PREFIX/nexus-server/data/templates/7ac36ad7-a960-4c87-916a-4c1068d19b59.tar
```

Feel free to repeat the operation above with as many templates as you like.

<!-- ============================================================================================================== -->
# Copy a template from another server

To copy a template from another nexus server, `rsync` must be used instead of `scp` as it supports sparse files. Sparse files are files where consecutive zeros are not actually stored in data blocks. Template files (qcow) might appear much bigger than they really are as `ls` doesn't show the real block usage. To display the real size of a file, use `du -h FILE` instead.

Use the following recipe to copy a template using `rsync` over ssh (requires ssh key pairs):
```sh
# On the destination server, create the template directory
mkdir /path_to_templates/4913a2bb-edfe-4dfe-af53-38197a44523b

# On the source server, copy the template directory to the destination server
rsync -P --sparse -r 4913a2bb-edfe-4dfe-af53-38197a44523b  destination_server:/path_to_templates/4913a2bb-edfe-4dfe-af53-38197a44523b
 ```

<!-- ============================================================================================================== -->
# Additional tools

The `genpwd` tools, located in `PREFIX/nexus-server/bin/`, can be used to obtain the hash of a password typed on the keyboard. This can be useful when wanting to update a password by directly editing the config file that stores encrypted user passwords (`PREFIX/nexus-server/config/users.json`).

<!-- ============================================================================================================== -->
# Building QEMU from source

1. Retrieve QEMU' source code from [https://www.qemu.org/download/](https://www.qemu.org/download/)
1. Decompress the archive and go into QEMU' source directory:
   ```sh
   tar vfxJ qemu-9.2.2.tar.xz
   cd qemu-9.2.2
   ```
1. Install the required dependencies:
   ```sh
   sudo apt-get install -y build-essential meson libspice-server-dev libattr1-dev libcap-ng-dev libusb-1.0-0-dev libusbredirparser-dev
   ```
1. Enable spice (for remote desktop), virtfs (for shared folders), libusb (for usb passthrough) and usb-redir (for usb redirection over the network) in QEMU's config:
   ```sh
   ./configure --enable-spice --enable-virtfs --enable-libusb --enable-usb-redir
   ```
1. Build QEMU:
   ```sh
   make
   ```
1. Install QEMU system-wide into `/usr/local/bin`:
   ```sh
   make install
   ```

<!-- ============================================================================================================== -->
# Building remote-viewer from source

1. Clone virt-viewer source code from gitlab:
   ```sh
   git clone git@gitlab.com:virt-viewer/virt-viewer.git virt-viewer.git
   ```
1. Install the required dependencies:
   ```sh
   sudo apt-get install -y meson ninja-build gtk+-3.0 libxml2-dev libsystemd-dev
   ```
1. Build remote-viewer (here in local directory `tmp`)
   ```sh
   meson setup build
   meson configure build -Ddefault_library=static -Ddebug=false -Dbuildtype=release -Dstrip=true -Dlibvirt=disabled -Dovirt=disabled -Dvnc=disabled -Dvte=disabled
   cd build
   meson compile remote-viewer
   ```

<!-- ============================================================================================================== -->
# Server software requirements

## Linux kernel with KVM

A Linux kernel with KVM is required. KVM is a module that provides a userspace API to access hardware-assisted virtualization instructions, thus "transforming" the Linux kernel into a hypervisor. To make sure the KVM module is loaded into the kernel, run:
```sh
lsmod|grep kvm
```
If you don't get an output similar to the one below, it means the KVM module is not loaded into the kernel. A possible cause for this behavior is if you didn't enable hardware virtualization support in your machine's BIOS/firmware. Note that if you have an AMD CPU, you should see `kvm_amd` in place of `kvm_intel`:
```sh
kvm_intel             294912  0
kvm                   782336  1 kvm_intel
```

## Required software

The following software is required to run nexus-server:

- QEMU system 6.0.0 or newer
- `qemu-img` to manipulate VM disk images
- `guestfish` to export/import files from VM disk images
- `uuidgen` to generate UUIDs from bash

Note that Ubuntu 20.04 ships with QEMU 4.2 which is too old, but Ubuntu 22.04 ships with QEMU 6.2 and Ubuntu 24.04 with QEMU 8.2.

The latest stable version of QEMU can be downloaded from [https://www.qemu.org/download/](https://www.qemu.org/download/). To build QEMU from source, read the [Building QEMU from source](##Building-QEMU-from-source) section at the end of this document.

To obtain, build, and install nexus-server, these additionnal software are required:

- git
- golang 1.22 or newer
- GNU make
- certtool

To install all the above-mentioned packages on a Ubuntu 24.04 and 22.04 system, run:
```sh
sudo apt-get install -y qemu-system-x86 qemu-utils guestfish uuid-runtime git golang-go make gnutls-bin
```

On Ubuntu 24.04, the following additionnal package must be installed:
```sh
sudo apt-get install -y qemu-system-modules-spice
```

## Note about Go compilers

To compile nexus-server, Go version 1.22 or above is required. Snap and flatpak package repositories provide recent enough versions.

Beware that Ubuntu 22.04 comes with go 1.18 which isn't recent enough. You can either use the flatpak or snap versions or install the official Go 1.22.7 toolchain from [https://go.dev/dl/](https://go.dev/dl/).

## Important: guestfish on Ubuntu

As mentioned above, `guestfish` is used to import or export files from VMs. Normally, it should be executed as a non-root user (as is done by nexus-server). However, on Ubuntu specifically, `guestfish` requires root access due to a side-effect of Ubuntu's wrong file permissions with `/boot/vmlinuz*` files ([more info here](https://libguestfs.org/guestfs-faq.1.html)). Consequently, restoring the correct file permissions is necessary for nexus-server to work properly:
```sh
sudo chmod 0644 /boot/vmlinuz*
```

However, each time a new kernel is installed, the permissions will be wrong again. Therefore, `chmod` must be executed every time a new kernel is installed to fix its permissions! When installing nexus-server for production, a cron job service is created which takes care of setting the right kernel permissions. However, this is not the case when installing the development version of nexus-server. In this case. you will have to manually change your kernel permissions as described above.

<!--
## Public key for client access

Make sure to transmit the public key (certificate) located in `PREFIX/nexus-server/certs/ca-cert.pem` to any nexus client as they will need this key to authentify themselves and communicate with the server.

## Updating a template

The `vm_run` script, located in `PREFIX/nexus-server/bin/`, is a convenience script that runs a VM on the specified image file(s). It can be used to update a template's disk image. Here is its usage:
```
vm_run - launch a VM on the specified disk image(s).
The VM is configured with 4 CPUs and 4G of RAM.

Usage: vm_run -d disk -i iso
disk    the disk image to use (typically a .qcow file)
iso     the CD/DVD-ROM image to use (typically an .iso file)

At least one image must be specified.
```

Before updating a template's disk image, **make absolutely sure it IS NOT used by any VMs**, otherwise all VMs based on this template will be corrupted!

## HOWTO: Exporting a VM disk image

Given each VM's disk is an overlay baked by a template's disk image, exporting a VM's disk image into a dedicated and independant file is slightly tricky. Below are the steps to do so.

Let:

- `base.qcow` be the template's disk image
- `overlay.qcow` be the VM's disk image (baked by `base.qcow`)

We therefore have the following relationship:

```
base.qcow <-- overlay.qcow
```

Steps:

1) Copy the overlay into a new one:
   ```sh
   cp overlay.qcow new_overlay.qcow
   ```
2) Rebase the new overlay in order to become a standalone disk file:
   ```sh
   qemu-img rebase -b "" new_overlay.qcow
   ```
3) Move the new overlay disk into the template directory:
   ```sh
   mv new_overlay.qcow $NEXUS_DATA_PATH/templates/my_new_template/
   ```

After completion of these 3 steps, `new_overlay.qcow` is equivalent to `base.qcow` + `overlay.qcow`.

Note that step 2) is very slow and can take several minutes given the template file can be very large (> 10GB).
-->
